#include "matrix.h"
#include <assert.h>

//start
// Allocates memory for a rows x cols array of double and 
// initializes each cell to zero.
Matrix:: Matrix(int rows, int cols):m_Rows(rows), m_Cols(cols) {
    m_NumArray = new double*[rows];
    for (int r = 0; r < rows; ++r) {
        m_NumArray[r]  = new double[cols];
        for(int c = 0; c < cols; ++c)
           m_NumArray[r][c]  = 0;
    }
}

// Refactoring utility function
void Matrix::sweepClean() {
   for (int r = 0; r < m_Rows; ++r)
      delete[] m_NumArray[r] ;
   delete[] m_NumArray;
}

Matrix::~Matrix() {
   sweepClean();
}


// Refactoring utility function
void Matrix::clone(const Matrix& mat) {
   m_NumArray = new double*[mat.m_Rows];
   for (int r = 0; r < mat.m_Rows; ++r) {
       m_NumArray[r] = new double[mat.m_Cols];
       for(int c = 0; c < mat.m_Cols; ++c)
            m_NumArray[r][c] = mat(r,c);
   }
}   
   
// Clones mat
Matrix::Matrix(const Matrix& mat) {
   clone(mat);
}

double& Matrix::operator()(int r, int c) {
   assert (r >= 0 && r < m_Rows && c >= 0 && c < m_Cols);
   return m_NumArray[r][c];
}

double Matrix::operator()(int r, int c) const {
   assert (r >= 0 && r < m_Rows && c >= 0 && c < m_Cols);
   return m_NumArray[r][c];   
}

// Deletes host content and then clones mat
Matrix& Matrix::operator=(const Matrix& mat) {
   sweepClean();
   clone(mat);
   return *this;
}

// matrix addition - assumes size compatibility
Matrix Matrix::operator+(const Matrix& mat) const {
   assert(m_Rows == mat.m_Rows && m_Cols == mat.m_Cols);
   Matrix mat2(*this);
   for (int r = 0; r < m_Rows; ++r)
      for(int c = 0; c < m_Cols; ++c)
           mat2(r,c) += mat(r,c);
   return mat2;
}

// Dot product of host row times other Matrix col.
// Assumes both have same size.
double Matrix::rcprod(int row, const Matrix& mat, int col) const {
   double res(0);
   for (int i = 0; i < m_Cols; ++i)
      res += m_NumArray[row][i] * mat(i, col);
   return res;
}

// matrix multiplication - assumes size compatibility
Matrix Matrix::operator*(const Matrix& mat) const {
   assert(m_Cols == mat.m_Rows);
   Matrix mat2(m_Rows, mat.m_Cols);
   for (int r = 0; r < m_Rows; ++r)
      for(int c = 0; c < mat.m_Cols; ++c)
           mat2(r,c) = rcprod(r, mat, c);
   return mat2;
}

bool Matrix::operator==(const Matrix& mat) const {
   if(m_Rows != mat.m_Rows || m_Cols != mat.m_Cols)
      return false;
   for (int r = 0; r < m_Rows; ++r)
      for(int c = 0; c < m_Cols; ++c)
         if(m_NumArray[r][c] != mat(r,c))
            return false;
   return true;
}

int Matrix::getRows() const {
   return m_Rows;
}

int Matrix::getCols() const {
   return m_Cols;
}

QString Matrix::toString() const {
   QString res;
   int pos;
   for (int r = 0; r < m_Rows; ++r) {
      for(int c = 0; c < m_Cols; ++c)
         res += QString("%1\t").arg(m_NumArray[r][c]);
      pos = res.lastIndexOf('\t');
      res.replace(pos, 1, '\n');
   }
   return res;
}
